/* PSPFTPD */

#include "std.h"
#include "util.h"
#include "ftp.h"
#include "ftpd.h"
#include "psp_init.h"
#include "psp_pg.h"
#include "psp_cfg.h"

#include <pspwlan.h>


#include "nlh.h" // net lib helper

//NOTE: kernel mode module flag and kernel mode thread are both required
PSP_MODULE_INFO("PSP-FTPD", 0x1000, 1, 1); /* 0x1000 REQUIRED!!! */
PSP_MAIN_THREAD_ATTR(0); /* 0 REQUIRED!!! */
PSP_MAIN_THREAD_STACK_SIZE_KB(32); /* smaller stack for kernel thread */

static void 
DoInetNetFtpd(void)
{
  int last_state;
  u32 err;
  char szMyIPAddr[32];
  int connectionConfig = -1;
  PICKER pickConn; // connection picker
  int iNetIndex;
  int iPick;


  // nlhInit must be called from user thread for DHCP to work
  err = nlhInit();

  if (err != 0) {
    pspDebugScreenPrintf("nlhInit: %d !\n", err);
    nlhTerm();
    return;
  }

  pgInit();

	while (sceWlanGetSwitchState() != 1)	//switch = off
	{
		pgPrint(0,0, "Turn the wifi switch on!", PG_TEXT_COLOR);
		sceKernelDelayThread(1000*1000);
	}

  // enumerate connections
  my_initpicker(&pickConn, "Choose a connection:                      X OK    [] CANCEL");

  for (iNetIndex = 1; iNetIndex < 100; iNetIndex++) // skip the 0th connection
  {
    char data[128];
    char name[128];
    char detail[128];
    if (sceUtilityCheckNetParam(iNetIndex) != 0) break;  // no more
    sceUtilityGetNetParam(iNetIndex, 0, name);

    sceUtilityGetNetParam(iNetIndex, 1, data);
    strcpy(detail, "SSID=");
    strcat(detail, data);

    sceUtilityGetNetParam(iNetIndex, 4, data);
    if (data[0]) {
      // not DHCP
      sceUtilityGetNetParam(iNetIndex, 5, data);
      strcat(detail, " IPADDR=");
      strcat(detail, data);
    } else {
      strcat(detail, " DHCP");
    }

    my_addpick(&pickConn, name, detail, (u32)iNetIndex);
    if (pickConn.pick_count >= MAX_PICK) break;  // no more
  }

  if (pickConn.pick_count == 0) {
    pgPrint(0,0, "No wifi connections, please create one !", PG_TEXT_COLOR);
    goto close_net;
  }

  pickConn.pick_start = 0;
  iPick = my_picker(&pickConn);
  if (iPick == -1) goto close_net;
  connectionConfig = (int)(pickConn.picks[iPick].userData);

connect_wifi:

  err = sceNetApctlConnect(connectionConfig);
  if (err != 0) {
    goto close_net;
  }

  {
    char buffer[128];
    pgClear();
    strcpy(buffer, "Trying to connect to ");
    strcat(buffer, pickConn.picks[iPick].szBig);
    pgPrint(0,0, buffer, PG_TEXT_COLOR);
  }

  // 4 connected with IP address
  
  last_state = -1;

  while (1) 
  {
    int state = 0;
    char buffer[128];

		sceKernelDelayThread(50*1000);
    err = sceNetApctlGetState(&state);

    if (err != 0)
    {
      pgPrint(0,1, "Connection failed !", PG_TEXT_COLOR);
      goto close_connection;
    }

    {
      SceCtrlData c;
      sceCtrlReadBufferPositive(&c, 1);
      if (c.Buttons & PSP_CTRL_SQUARE) goto close_connection;
    }

    if (state != last_state) 
    {
			if (last_state == 2 && state == 0)
			{
				pgPrint(0,2, "Connecting to wifi Failed, Retrying...", PG_TEXT_COLOR);
				sceKernelDelayThread(500*1000); // 500ms
				goto connect_wifi;
			}
     
			sprintf(buffer, "Connection state %d of 4", state);
			pgPrint(0,1, buffer, PG_TEXT_COLOR);
			last_state = state;
		}

    // 0 - idle
    // 1,2 - starting up
    // 3 - waiting for dynamic IP
    // 4 - got IP - usable
    if (state == 4) break;  // connected with static IP
  }

  // get IP address
  if (sceNetApctlGetInfo(8, szMyIPAddr) != 0) {
    strcpy(szMyIPAddr, "unknown IP address");
  }

  ftpdLoop(szMyIPAddr);

close_connection:
  err = sceNetApctlDisconnect();

close_net:
  pgClear();
  pgPrint(0,0, "Closing connection ...", PG_TEXT_COLOR);
  
  nlhTerm();
  sceKernelDelayThread(1000000); 
}

void
user_thread(SceSize args, void *argp)
{
  DoInetNetFtpd();
}

int 
main(int argc, char **argv)
{
  int user_thid;

  pspDebugScreenInit();

  psp_init_stuff(argc, argv);

  pspDebugScreenPrintf("PSP-FTPD v0.3.6 by Zx-81\n\n");
  pspDebugScreenPrintf("Design by Cortes the Killer\n");
  pspDebugScreenPrintf("Based on the work of Raf, Krazy, Pet and Theno\n\n");
  pspDebugScreenPrintf("Please wait now loading !\n");
  sceKernelDelayThread(2000000); 

  if (nlhLoadDrivers(&module_info) != 0) {
    pspDebugScreenPrintf("unable to load wifi drivers !");
    return 0;
  }

  psp_read_config();

  pspDebugScreenPrintf("Launch main thread !\n");
  sceKernelDelayThread(1000000); 

  user_thid = sceKernelCreateThread( "user_thread", 
     (SceKernelThreadEntry)user_thread, 0x8, 256*1024, 
     PSP_THREAD_ATTR_USER, 0 );
  if(user_thid >= 0) {
    sceKernelStartThread(user_thid, 0, 0);
    sceKernelWaitThreadEnd(user_thid, NULL);
  }

  sceKernelExitGame();

  psp_exit(0);

  return 0;
}

